"""Simple OS Independent 'cat' Command-line Tool made in Python."""

__project__ = 'cat_win'
__version__ = '1.4.0'
__sysversion__ = '3.11.2 (main, Feb  7 2023, 16:38:35) [MSC v.1934 64 bit (AMD64)]'
__author__ = 'Silas A. Kraume'
__url__ = 'https://github.com/SilenZcience/cat_win'
